"""Main file for pagey."""

import os

from .args import *
from .pagerduty import *
from .defaults import DEF_NAME, DEF_DESC, DEF_VERSION, DEF_GITHUB
from .slack import *

COMMANDS = ["oncall", "info"]


def main() -> None:
    """Run main entrypoint."""
    # Parse command line arguments
    get_args()

    # Ensure environment tokens are present
    try:
        SLACK_TOKEN = os.environ["PAGEY_SLACK_TOKEN"]
    except KeyError:
        print("Error, env variable 'PAGEY_SLACK_TOKEN' not set", file=sys.stderr)
        sys.exit(1)
    try:
        PD_TOKEN = os.environ["PAGEY_PD_TOKEN"]
    except KeyError:
        print("Error, env variable 'PAGEY_PD_TOKEN' not set", file=sys.stderr)
        sys.exit(1)

    # Initialize Pagerduty module
    pagerduty = PageyPD(PD_TOKEN)

    def commandCallback(command: str) -> str:
        """This is a callback function for Slack to evaluate response based on given command.

        Args:
            command (str): the command/message after the bot mention (e.g.: @pagey <command>).

        Returns:
            str: The reply to be sent to Slack.
        """
        # [Command: oncall] Get Pagerduty schedules
        if command.startswith("oncall"):
            schedules = pagerduty.get_schedules()
            response = ""
            for team, users in schedules.items():
                response += f"*{team}*\n"
                # Sort by escalation level
                users.sort(key=lambda s: s["level"])
                for user in users:
                    if int(user["level"]) == 1:
                        response += (
                            f"* [lvl: *{user['level']}* -> {user['until']}] *{user['name']}*\n"
                        )
                    else:
                        response += (
                            f"* [lvl: *{user['level']}* -> {user['until']}] {user['name']}\n"
                        )
                response += "\n"
            return response
        # [Command: info] Report some info
        if command.startswith("info"):
            return f"{DEF_NAME} ({DEF_VERSION}) - {DEF_DESC}\nFind me here: {DEF_GITHUB}\n"

        return "Available commands: " + ", ".join(COMMANDS)

    # Connect to Slack (RTM mode)
    slack = PageySlack(SLACK_TOKEN, commandCallback)
    if not slack.connect():
        print("Connection to Slack failed. Exception traceback printed above.", file=sys.stderr)
        sys.exit(1)
    print("Pagey connected to Slack and running!")
    slack.run()


if __name__ == "__main__":
    main()



"""Pagerduty module."""

from typing import List, Dict, Any
import datetime
import json
import sys

from dateutil import parser
import requests


class PageyPD:
    """Pagey PagerDuty Class."""

    # --------------------------------------------------------------------------
    # Contrcutor
    # --------------------------------------------------------------------------
    def __init__(self, token: str) -> None:
        self.__token = token

    # --------------------------------------------------------------------------
    # Public Functions
    # --------------------------------------------------------------------------
    def get_schedules(self) -> Dict[str, Any]:
        """Fetch oncall schedules from Pagerduty API."""
        url = "https://api.pagerduty.com/oncalls"
        now = datetime.datetime.now()
        data = {}  # type: Dict[str, Any]

        for item in self.__get_api_response(url, "oncalls"):
            if item["start"] and item["end"]:
                date_start = parser.parse(item["start"])
                date_end = parser.parse(item["end"])
                if (
                    now.timestamp() > date_start.timestamp()
                    and now.timestamp() < date_end.timestamp()
                ):
                    team_name = item["escalation_policy"]["summary"]
                    if team_name not in data:
                        data[team_name] = []
                    data[team_name].append(
                        {
                            "name": item["user"]["summary"],
                            "level": item["escalation_level"],
                            "until": f"{date_end:%Y-%m-%d}",
                        }
                    )
        return data

    # --------------------------------------------------------------------------
    # Private Functions
    # --------------------------------------------------------------------------
    def __get_api_response(self, url: str, ret_key: str) -> List[Dict[str, Any]]:
        """Make generic GET requests against Pagerduty API and return its response.

        Args:
            url (str): API URL to request against.
            ret_key (str): The top-level JSON key for which to return its childs.

        Returns:
            list: Returns list of response dicts.
        """
        # Required request headers for PD APIv2
        headers = {
            "Accept": "application/vnd.pagerduty+json;version=2",
            "Content-Type": "application/json",
            "Authorization": f"Token token={self.__token}",
        }
        params = {}
        limit = 20
        offset = 0
        data = []  # type: List[Dict[str, Any]]

        while True:
            params = {
                "limit": limit,
                "offset": offset,
            }
            try:
                response = requests.request(
                    "GET",
                    url,
                    params=params,
                    headers=headers,
                    timeout=30,
                )
                val = json.loads(response.content.decode("utf-8"))
                data = data + val[ret_key]  # concatenate result list
                # Do we paginate?
                if val["more"]:
                    offset += limit
                    continue
                break
            except requests.exceptions.URLRequired as url_err:
                print(str(url_err), file=sys.stderr)
                break
            except requests.exceptions.HTTPError as http_err:
                print(str(http_err), file=sys.stderr)
                break
            except requests.exceptions.TooManyRedirects as redir_err:
                print(str(redir_err), file=sys.stderr)
                break
            except requests.exceptions.ConnectTimeout as conn_time_err:
                print(str(conn_time_err), file=sys.stderr)
                break
            except requests.exceptions.ReadTimeout as read_time_err:
                print(str(read_time_err), file=sys.stderr)
                break
            except requests.exceptions.Timeout as time_err:
                print(str(time_err), file=sys.stderr)
                break
            except requests.exceptions.ConnectionError as conn_err:
                print(str(conn_err), file=sys.stderr)
                break
            except requests.exceptions.RequestException as req_err:
                print(str(req_err), file=sys.stderr)
                break
        # close response object after done
        # TODO: add try-except
        response.close()

        return data



"""Pip configuration."""
# https://github.com/pypa/sampleproject/blob/main/setup.py

from setuptools import setup

with open("README.md", "r") as fp:
    long_description = fp.read()

with open("requirements.txt", "r") as fp:
    requirements = fp.read()

setup(
    name="pagey",
    version="0.1.0",
    packages=[
        "pagey",
        "pagey.pagerduty",
        "pagey.slack",
    ],
    entry_points={
        "console_scripts": [
            # cmd = package[.module]:func
            "pagey=pagey:main",
        ],
    },
    install_requires=requirements,
    description="Pagey is a Pagerduty slack bot.",
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=["pagerduty", "slack"],
    author="cytopia",
    author_email="cytopia@everythingcli.org",
    url="https://github.com/Flaconi/slackbot-pagey",
    project_urls={
        "Source Code": "https://github.com/Flaconi/slackbot-pagey",
        "Bug Tracker": "https://github.com/Flaconi/slackbot-pagey",
    },
    python_requires=">=3.6",
    classifiers=[
        # https://pypi.org/classifiers/
        #
        # How mature is this project
        "Development Status :: 3 - Alpha",
        # How does it run
        "Environment :: Console",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        # License
        "License :: OSI Approved :: MIT License",
        # Where does it run
        "Operating System :: OS Independent",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        # Project topics
        "Topic :: Internet",
        "Topic :: Security",
        "Topic :: System :: Systems Administration",
        "Topic :: Utilities",
        # Typed
        "Typing :: Typed",
    ],
)



"""Parse command line arguments."""

import argparse

from .defaults import DEF_NAME, DEF_DESC, DEF_VERSION, DEF_GITHUB


def _get_version() -> str:
    """Return version information."""
    return """%(prog)s: Version %(version)s
(%(url)s)""" % (
        {"prog": DEF_NAME, "version": DEF_VERSION, "url": DEF_GITHUB}
    )


def get_args() -> argparse.Namespace:
    """Retrieve command line arguments."""
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawTextHelpFormatter,
        add_help=False,
        usage="""%(prog)s
       %(prog)s -v, --version
       %(prog)s -h, --help"""
        % ({"prog": DEF_NAME}),
        description=DEF_DESC
        + """

IMPORTANT:
  You will have to export PAGEY_SLACK_TOKEN and PAGEY_PD_TOKEN
  to your environment.

  export PAGEY_SLACK_TOKEN="read-write slack token"
  export PAGEY_PD_TOKEN="read-only pagerduty token"
""",
    )

    misc = parser.add_argument_group("misc arguments")

    misc.add_argument(
        "-v",
        "--version",
        action="version",
        version=_get_version(),
        help="Show version information and exit.",
    )
    misc.add_argument("-h", "--help", action="help", help="Show this help message and exit.")

    # Return arguments
    return parser.parse_args()
